#!/usr/bin/python
# -*- coding: UTF-8 -*-
"""一个相片日期處理程式

依賴: python-pyexiv2 imagemagick
"""

import gtk, gobject
import os
import cPickle as pickle
import pyexiv2, datetime

try: import i18n
except: from gettext import gettext as _

__version__ = '0.0.1'

__usage__ = _('''LiliDatetime

Usage:
  gedit [OPTION...] [FILE...]

Options:
  -h, --help                     Show help options
  -v, --version                  Show version information
''')

def gtkopen(title='', parent=None, 
        patterns=[], mimes=[], name_mimes={}, name_patterns={}, folder=None):
    """Displays an open dialog.
    
    return the  full path , or None"""
    filechooser = gtk.FileChooserDialog(title or _('Open'),
                    parent,
                    gtk.FILE_CHOOSER_ACTION_OPEN,
                    (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                    gtk.STOCK_OPEN, gtk.RESPONSE_OK))

    if patterns:
        file_filter = gtk.FileFilter()
        for pattern in patterns:
            file_filter.add_pattern(pattern)
        filechooser.set_filter(file_filter)
        pass
    if mimes:
        file_filter = gtk.FileFilter()
        for mime in mimes:
            file_filter.add_mime_type(mime)
        filechooser.add_filter(file_filter)
        pass
    if name_mimes:
        for name, mime in name_mimes.iteritems():
            file_filter = gtk.FileFilter()
            file_filter.set_name(name)
            file_filter.add_mime_type(mime)
            filechooser.add_filter(file_filter)
    if not "*" in name_patterns.values():
        name_patterns[_("所有文件")] = "*"
    for name, pattern in name_patterns.iteritems():
            file_filter = gtk.FileFilter()
            file_filter.set_name(name)
            file_filter.add_pattern(pattern)
            filechooser.add_filter(file_filter)

    filechooser.set_default_response(gtk.RESPONSE_OK)

    if folder:
        filechooser.set_current_folder(folder)
        
    response = filechooser.run()
    if response != gtk.RESPONSE_OK:
        filechooser.destroy()
        return
    
    path = filechooser.get_filename()
    if path and os.access(path, os.R_OK):
        filechooser.destroy()
        return path
        
    abspath = os.path.abspath(path)

    error(_('Could not open file "%s"') % abspath,
        _('The file "%s" could not be opened. '
        'Permission denied.') %  abspath)

    filechooser.destroy()
    return path

windows = []

class MainWindow:

    def readconfig(self, *args):
        self.profdir = os.environ['HOME'] + '/.config/lilidatetime'
        self.profile = self.profdir + '/lilidatetime.pickle'
        try:
            os.makedirs(self.profdir)
        except:
            pass
        try:
            prof = pickle.load(file(self.profile))
        except:
            prof = {}
        self.font    = prof.get('font', "Sans 18")
        self.color   = prof.get('color', "#F4A862")
        self.gravity = prof.get('gravity', "SouthEast")
        self.stroke  = prof.get('stroke', "fill")
        self.margin  = prof.get('margin', "10,10")
        pass

    def writeconfig(self, *args):
        prof = {}
        prof['font']    = self.font
        prof['color']   = self.color
        prof['gravity'] = self.gravity
        prof['stroke']  = self.stroke
        prof['margin']  = self.margin
        pickle.dump(prof, file(self.profile, 'w'))
        pass

    def __init__(self, url = None, create = True, accel_group = None, tooltips = None):
        windows.append(self)
        self.text = "2007:11:11 16:00:10"
        self.url = url or None
        if accel_group is None:
            self.accel_group = gtk.AccelGroup()
        else:
            self.accel_group = accel_group
        if tooltips is None:
            self.tooltips = gtk.Tooltips()
        else:
            self.tooltips = tooltips
        self.tooltips.enable()
        if create:
            self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
            self.window.set_title(_(""))
            self.window.set_position(gtk.WIN_POS_NONE)
            self.window.add_accel_group(self.accel_group)
            self.window.set_default_size(500, 500)
            self.window.set_icon_name("gnome-mime-image")
            self.window.set_title(_("相片日期處理"))
            self.window.connect("delete_event", self.on_quit)
            self.window.show_all()

        self.vbox1 = gtk.VBox(False, 0)
        self.vbox1.show()
        self.vbox1.set_spacing(0)

        self.readconfig()

        self.menubar1 = gtk.MenuBar()
        self.menubar1.show()

        self.menuitem1 = gtk.MenuItem(_("檔案(_F)"))
        self.menuitem1.show()

        self.menuitem1_menu = gtk.Menu()

        self.imagemenuitem2 = gtk.ImageMenuItem(_("gtk-open"))
        self.imagemenuitem2.show()
        self.imagemenuitem2.connect("activate", self.on_open)

        self.menuitem1_menu.append(self.imagemenuitem2)

        self.imagemenuitem3 = gtk.ImageMenuItem(_("gtk-save"))
        self.imagemenuitem3.show()
        self.imagemenuitem3.connect("activate", self.on_save)

        self.menuitem1_menu.append(self.imagemenuitem3)

        self.imagemenuitem4 = gtk.ImageMenuItem(_("gtk-save-as"))
        #self.imagemenuitem4.show()
        self.imagemenuitem4.connect("activate", self.on_saveas)

        self.menuitem1_menu.append(self.imagemenuitem4)

        self.separatormenuitem1 = gtk.MenuItem()
        self.separatormenuitem1.show()

        self.menuitem1_menu.append(self.separatormenuitem1)

        self.imagemenuitem5 = gtk.ImageMenuItem(_("gtk-quit"))
        self.imagemenuitem5.show()
        self.imagemenuitem5.connect("activate", self.on_quit)

        self.menuitem1_menu.append(self.imagemenuitem5)

        self.menuitem1.set_submenu(self.menuitem1_menu)

        self.menubar1.append(self.menuitem1)

        self.menuitem4 = gtk.MenuItem(_("說明(_H)"))
        self.menuitem4.show()

        self.menuitem4_menu = gtk.Menu()

        self.imagemenuitem10 = gtk.ImageMenuItem(_("gtk-about"))
        self.imagemenuitem10.show()
        self.imagemenuitem10.connect("activate", self.on_about)

        self.menuitem4_menu.append(self.imagemenuitem10)

        self.menuitem4.set_submenu(self.menuitem4_menu)

        self.menubar1.append(self.menuitem4)

        self.vbox1.pack_start(self.menubar1, False, True, 0)

        self.scrolledwindow1 = gtk.ScrolledWindow()
        self.scrolledwindow1.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        self.scrolledwindow1.show()
        self.scrolledwindow1.set_shadow_type(gtk.SHADOW_NONE)

        self.viewport1 = gtk.Viewport()        
        self.viewport1.show()

        self.image = gtk.Image()
        self.image.set_padding(0, 0)
        self.image.show()
        self.viewport1.add(self.image)

        self.scrolledwindow1.add(self.viewport1)

        self.vbox1.pack_start(self.scrolledwindow1, True, True, 0)

        self.hbox2 = gtk.HBox(False, 0)
        self.hbox2.show()
        self.hbox2.set_spacing(0)

        self.button3 = gtk.Button(None, gtk.STOCK_OPEN)
        self.tooltips.set_tip(self.button3, _("打開相片"))
        self.button3.show()
        self.button3.set_relief(gtk.RELIEF_NORMAL)
        self.button3.connect("clicked", self.on_open)

        self.hbox2.pack_start(self.button3, False, False, 0)

        self.hseparator2 = gtk.HSeparator()
        self.hseparator2.show()
        self.hbox2.pack_start(self.hseparator2, True, True, 10)

        self.fontbutton = gtk.FontButton()
        self.tooltips.set_tip(self.fontbutton, _("日期字型"))
        self.fontbutton.show()
        self.fontbutton.set_font_name(self.font)
        self.fontbutton.connect("font_set", self.on_font)
        self.hbox2.pack_start(self.fontbutton, False, False, 0)

        # self.colorbutton of GtkColorButton should be here
        self.colorbutton = gtk.ColorButton()
        self.colorbutton.show()
        self.colorbutton.set_tooltip_text( _("日期颜色"))
        self.colorbutton.set_color(gtk.gdk.color_parse(self.color))
        self.colorbutton.connect("color-set", self.on_color)
        self.hbox2.pack_start(self.colorbutton, False, False, 0)

        self.comboboxentry5_List = gtk.ListStore(gobject.TYPE_STRING)
        self.comboboxentry5 = gtk.ComboBoxEntry()
        self.comboboxentry5.set_size_request(120, -1)
        self.comboboxentry5.set_tooltip_text(_("日期位置"))
        self.comboboxentry5.show()
        for i in ["SouthEast", "NorthEast", "SouthWest", "NorthWest"]:
            self.comboboxentry5_List.append([i])

        self.comboboxentry_entry5 = self.comboboxentry5.child
        self.comboboxentry_entry5.set_text(self.gravity)
        self.comboboxentry_entry5.set_editable(True)
        self.comboboxentry_entry5.show()
        self.comboboxentry_entry5.set_visibility(True)

        self.comboboxentry5.set_model(self.comboboxentry5_List)
        self.comboboxentry5.set_text_column(0)
        self.comboboxentry5.connect("changed", self.on_gravity)
        self.hbox2.pack_start(self.comboboxentry5, False, False, 0)

        self.comboboxentry4_List = gtk.ListStore(gobject.TYPE_STRING)
        self.comboboxentry4 = gtk.ComboBoxEntry()
        self.comboboxentry4.set_tooltip_text(_("笔触"))
        self.comboboxentry4.set_size_request(80, -1)
        for i in ["stroke", "fill"]:
            self.comboboxentry4_List.append([i])        
        self.comboboxentry4.show()

        self.comboboxentry_entry4 = self.comboboxentry4.child
        self.comboboxentry_entry4.set_text(self.stroke)
        self.comboboxentry_entry4.set_editable(True)
        self.comboboxentry_entry4.show()
        self.comboboxentry_entry4.set_visibility(True)

        self.comboboxentry4.set_model(self.comboboxentry4_List)
        self.comboboxentry4.set_text_column(0)
        self.comboboxentry4.connect("changed", self.on_stroke)
        self.hbox2.pack_start(self.comboboxentry4, False, False, 0)

        self.comboboxentry6_List = gtk.ListStore(gobject.TYPE_STRING)
        self.comboboxentry6 = gtk.ComboBoxEntry()
        self.comboboxentry6.set_tooltip_text(_("邊距"))
        self.comboboxentry6.set_size_request(80, -1)
        self.comboboxentry6.show()
        for i in ["0,0", "5,5", "10,10", "12,12", "15, 15", "20,20", "30, 30", "40, 40", "50,50"]:
            self.comboboxentry6_List.append([i])

        self.comboboxentry_entry6 = self.comboboxentry6.child
        self.comboboxentry_entry6.set_text("10,10")
        self.comboboxentry_entry6.set_editable(True)
        self.comboboxentry_entry6.show()
        self.comboboxentry_entry6.set_visibility(True)

        self.comboboxentry6.set_model(self.comboboxentry6_List)
        self.comboboxentry6.set_text_column(0)
        self.comboboxentry6.connect("changed", self.on_margin)
        self.hbox2.pack_start(self.comboboxentry6, False, False, 0)

        self.hseparator1 = gtk.HSeparator()
        self.hseparator1.show()
        self.hbox2.pack_start(self.hseparator1, True, True, 10)

        self.button4 = gtk.Button(None, gtk.STOCK_APPLY)
        self.tooltips.set_tip(self.button4, _("預覽"))
        self.button4.show()
        self.button4.set_relief(gtk.RELIEF_NORMAL)
        self.button4.connect("clicked", self.on_apply)

        self.hbox2.pack_start(self.button4, False, False, 0)

        self.button5 = gtk.Button(None, gtk.STOCK_SAVE)
        self.tooltips.set_tip(self.button5, _("保存"))
        self.button5.show()
        self.button5.set_relief(gtk.RELIEF_NORMAL)
        self.button5.connect("clicked", self.on_save)

        self.hbox2.pack_start(self.button5, False, False, 0)

        self.vbox1.pack_start(self.hbox2, False, False, 0)


        if create:
            self.window.add(self.vbox1)
            pass

        if self.url:
            self.load()
            self.on_apply()
            pass

        pass

    def load(self, *args):
        self.image.set_from_file(self.url)
        image = pyexiv2.Image(self.url)
        self.window.set_title(self.url + ' - ' + _("相片日期處理"))
        image.readMetadata()
        try:
            self.datetime = image['Exif.Image.DateTime']
        except:
            self.datetime = str(datetime.datetime.today())

    def on_quit(self, widget, *args):
        #print 'on_quit:'
        #gtk.main_quit()
        windows.remove(self)
        self.window.destroy()
        if not windows: gtk.main_quit()
        pass

    def on_open(self, *args):
        #print 'on_open:'
        url = gtkopen(title=_('打开相片'), name_mimes={"相片和图片":"image/*"})
        if self.url:
            MainWindow(url)
            return
        if not url: return
        #print url
        self.url = url
        self.load()
        pass

    def on_save(self, widget, *args):
        #print 'on_save:'
        self.on_apply()
        os.popen2(['cp', '-av', self.jpg2, self.jpg1])
        pass

    def on_saveas(self, widget, *args):
        #print 'on_saveas:'
        pass

    def on_about(self, widget, *args):
        #print 'on_about:'
        authors = [
            #"Jiahua Huang <jhuangjiahua(at)gmail.com>",
            "Lili <xiehuoli@gmail.com>",
            ]
        about = gobject.new(gtk.AboutDialog, 
                name=_("相片日期处理"), 
                program_name=_("相片日期处理"),
                logo_icon_name="gnome-mime-image",
                version="0.1", 
                #copyright=_("(C) hiweed.com"),
                comments=_("将 Exif 日期打印到相片"),
                license="LGPLv3",
                #website="http://www.hiweed.com/",
                #website_label="hiweed.com",
                authors=authors)
        about.set_transient_for(self.window)
        about.run()     
        about.destroy()
        pass

    def on_font(self, widget, *args):
        self.font = widget.get_font_name()
        #print 'on_font:', self.font
        self.on_apply()
        pass

    def on_color(self, widget, *args):
        self.color = widget.get_color().to_string()
        #print 'on_color:', self.color
        self.on_apply()
        pass

    def on_gravity(self, widget, *args):
        self.gravity = widget.child.get_text()
        #print 'on_gravity:', self.gravity
        self.on_apply()
        pass

    def on_stroke(self, widget, *args):
        self.stroke = widget.child.get_text()
        #print 'on_stroke:', self.stroke
        self.on_apply()
        pass

    def on_margin(self, widget, *args):
        self.margin = widget.child.get_text()
        #print 'on_margin:', self.margin
        self.on_apply()
        pass

    def on_apply(self, *args):
        #print 'on_apply:'
        if not self.url:
            self.on_open()
        self.fontsize = self.font.split()[-1]
        ##print self.fontsize, self.color, self.gravity, self.stroke, self.margin
        self.text = self.datetime
        self.jpg1 = self.url or 'tes.jpg'
        self.jpg2 = '/dev/shm/lilidatetime'
        cmd = ['convert', self.jpg1, 
                  '-fill', 'grey', '-background', 'none', 
                  '-font', 'Sans', 
                  '-pointsize', self.fontsize,
                  '-gravity', self.gravity,
                  '-'+self.stroke, self.color,
                  '-draw', 'text '+self.margin+' "%s"'%self.text,
                  self.jpg2,
              ]
        #print ' '.join(cmd)
        if self.url:
            #print cmd
            os.popen2(cmd)[1].read()
            self.image.set_from_file(self.jpg2)
            self.writeconfig()
        pass


def main():
    import getopt, os, sys
    try:
        opts, args = getopt.getopt(sys.argv[1:], 'vh', ['version', 'help'])
        pass
    except:
        print __usage__
        return
    for o, v in opts:
        if o in ('-h', '--help'):
            print __usage__
            return
        elif o in ('-v', '--version'):
            print __version__
            return
        pass
    files = [ i for i in args if os.path.isfile(i) ]
    for i in files:
        MainWindow(i)
        pass
    if not windows:
        win = MainWindow()
        pass
    return gtk.main()
    
if __name__ == '__main__':
    main()

